import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';
import { GUI } from './jsm/libs/dat.gui.module.js';

class App {
    constructor() {
        const domWebGL = document.createElement('div');
        document.body.appendChild(domWebGL);
        this._domWebGL = domWebGL;

        const renderer = new THREE.WebGLRenderer({ antialias: true });
        //renderer.outputEncoding = THREE.sRGBEncoding;
        renderer.setPixelRatio(window.devicePixelRatio);
        //renderer.shadowMap.enabled =  true;
        domWebGL.appendChild(renderer.domElement);

        this._renderer = renderer;

        const scene = new THREE.Scene();
        //scene.background = new THREE.Color(0x050505);
        this._scene = scene;

        this._clock = new THREE.Clock();

        this._setupCamera();
        //this._setupLight();
        this._setupModel();
        //this._setupInteractions();
        this._setupGUI();
        //this._setupControls();
        //this._setupHelper();

        this._targetQuaternion = new THREE.Quaternion();
        this._generateTarget();
        
        window.onresize = this.resize.bind(this);

        this.resize();
        this.render()
    }

    _setupInteractions() {
        //.
    }

    _setupPostprocessing() {
        //.
    }

    _generateTarget() {
        const spherical = new THREE.Spherical();
        spherical.theta = Math.random() * Math.PI * 2;
        spherical.phi = Math.acos((2*Math.random()) - 1);
        spherical.radius = 2;

        this._target.position.setFromSpherical(spherical);

        const rotationMatrix = new THREE.Matrix4();
        rotationMatrix.lookAt(this._target.position, this._mesh.position, this._mesh.up);
        this._targetQuaternion.setFromRotationMatrix(rotationMatrix);
        
        setTimeout(this._generateTarget.bind(this), 2000);

    }

    _setupModel() {
        const geometry = new THREE.ConeGeometry(0.1, 0.5, 8);
        geometry.rotateX(Math.PI * 0.5);
        const material = new THREE.MeshNormalMaterial();
        const mesh = new THREE.Mesh(geometry, material);
        this._scene.add(mesh);

        const targetGeometry = new THREE.SphereGeometry(0.05);
        const targetMaterial = new THREE.MeshBasicMaterial({ color: 0xff0000 });
        const target = new THREE.Mesh(targetGeometry, targetMaterial);
        this._scene.add(target);

        const sphereGeometry = new THREE.SphereGeometry(2, 32, 32);
        const sphereMaterial = new THREE.MeshBasicMaterial(
            {
                color: 0xcccccc, 
                wireframe: true, 
                transparent: true, 
                opacity: 0.3 
            }
        );
        const sphere = new THREE.Mesh(sphereGeometry, sphereMaterial);
        this._scene.add(sphere);

        this._target = target;
        this._mesh = mesh;
    }

    _setupHelper() {
        //.
    }

    _setupGUI() {
        const stats = new Stats();
        this._domWebGL.appendChild(stats.dom);
        this._stats = stats;
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(70, window.innerWidth / window.innerHeight, 0.01, 10);
        camera.position.z = 5;

        this._camera = camera;
    }

    _setupControls() {
        //.
    }

    _setupLight() {
        //.
    }

    update() {
        const delta = this._clock.getDelta();

        if(!this._mesh.quaternion.equals(this._targetQuaternion)) {
            const speed = 2;
            const step = speed * delta;
            this._mesh.quaternion.rotateTowards(this._targetQuaternion, step);
        }
        
        this._stats.update();
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }
}

window.onload = function () {
    new App();
}